@@ -6,6 +6,7 @@
 :license: MIT, see LICENSE for more details.
 
 """
+
 import sys
 
 if sys.version_info >= (3, 8):
@@ -20,6 +21,8 @@ except PackageNotFoundError:
     # package is not installed
     pass
 
+# noqa: we need to import so that __init_subclass__() is executed once
+from . import attributes  # noqa: F401
 from .highlevel import PyVisaLibrary
 
 WRAPPER_CLASS = PyVisaLibrary
